//-*-c++-*-
#ifndef INCLUDED_SoundManagerMsg_h
#define INCLUDED_SoundManagerMsg_h

class RCRegion;

//! A small header that preceeds data sent by SoundManager between processes
struct SoundManagerMsg {
	//! the type to use when referring to Sounds
	typedef unsigned short Snd_ID;
	//! maintains even word alignment
	static const unsigned int MSG_SIZE=16;

	//! constructor
	SoundManagerMsg() : type(unknown), id(static_cast<Snd_ID>(-1)), region(NULL) {}

	//! virtual destructor
	/*! doesn't do anything, but don't remove it, otherwise this would no longer be a virtual base class */
	virtual ~SoundManagerMsg() {}

	//! Accessor for the id number, set by SoundManager
	Snd_ID getID() const { return id; }

private:
	friend class SoundManager;

	//! Denotes what type of message this is
	enum MsgType { add, del, wakeup, unknown } type;
	
	//! The id of the sound this is in reference to
	Snd_ID id;

	//! The RCRegion to free, if it's a deletion
	RCRegion * region;

	//! Sets up the header as an add message
	void setAdd(Snd_ID sndid) {
		type=add;
		id=sndid;
	}

	//! Sets up the header as an erase message
	void setDelete(RCRegion* rcregion) {
		type=del;
		region=rcregion;
	}

	//! Sets up the header as a wakeup message
	void setWakeup() {
		type=wakeup;
	}

	SoundManagerMsg(const SoundManagerMsg&); //!< don't call
	SoundManagerMsg operator=(const SoundManagerMsg&); //!< don't call
};

/*! @file
 * @brief Defines SoundManagerMsg, a small header used by SoundManager for sending messages between processes
 * @author ejt (Creator)
 *
 * $Author: ejt $
 * $Name: tekkotsu-1_3 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $Date: 2003/04/09 04:48:19 $
 */

#endif // INCLUDED_SoundManagerMsg_h
